import {
  CMType,
  HMType,
  HostMessage,
  Transaction,
  TransactionResponse,
  ValidationError,
} from '../types';
import {
  MessageCallback,
  MessageEventManager,
} from '../utils/MessageEventManager';

/**
 * An embedded Notabene component
 * @public
 */
export default class EmbeddedComponent {
  private _url: string;
  private _value: Partial<Transaction>;
  private _errors: ValidationError[] = [];

  private iframe?: HTMLIFrameElement;
  private eventManager: MessageEventManager;
  private ready = false;

  constructor(url: string, value: Partial<Transaction>) {
    this._url = url;
    this._value = value;
    this.eventManager = new MessageEventManager();
    this.on(CMType.INVALID, (message) => {
      if (message.type === CMType.INVALID) {
        this._errors = message.errors;
        this._value = message.value as Transaction;
      }
    });
    this.on(CMType.RESIZE, (message) => {
      if (message.type === CMType.RESIZE) {
        if (this.iframe) this.iframe.style.height = `${message.height}px`;
      }
    });
  }

  public get url(): string {
    return this._url;
  }

  public get value(): Partial<Transaction> {
    return this._value;
  }

  public get errors(): ValidationError[] {
    return this._errors;
  }

  public open() {
    document.location.href = this.url;
  }

  mount(parentId: string) {
    this.iframe = document.createElement('iframe');
    this.iframe.src = this.url;
    this.iframe.style.width = '100%';
    this.iframe.style.height = '100px';
    this.iframe.style.border = 'none';
    this.iframe.style.overflow = 'hidden';

    window.addEventListener('message', (event) => {
      if (event.source !== this.iframe?.contentWindow) {
        return;
      }
      console.log('received message from iframe', event.data);
      this.eventManager?.setPort(event.ports[0]);
      this.ready = true;
    });

    const parent = document.querySelector(parentId);
    if (!parent) throw new Error(`parentID ${parentId} not found`);
    parent.appendChild(this.iframe);
    this.iframe?.contentWindow?.focus();
  }

  send(message: HostMessage): void {
    this.eventManager.send(message);
  }

  on(messageType: string, callback: MessageCallback): void {
    this.eventManager.on(messageType, callback);
  }

  off(messageType: string, callback: MessageCallback): void {
    this.eventManager.off(messageType, callback);
  }

  update(value: Partial<Transaction>) {
    this._value = value;
    this.send({ type: HMType.UPDATE, value });
  }

  completion(): Promise<TransactionResponse> {
    return new Promise((resolve, reject) => {
      this.on(CMType.COMPLETE, (message) => {
        resolve((message as any).response);
      });
      this.on('error', (message) => {
        reject(new Error((message as any).message));
      });
    });
  }
}
