import EmbeddedComponent from './components/EmbeddedComponent';
import type {
  Agent,
  AgentType,
  CallbackOptions,
  ComponentMessage,
  Counterparty,
  DeclarationProof,
  Destination,
  HostMessage,
  IVMS101,
  MicroTransferProof,
  OwnershipProof,
  PersonType,
  ProofStatus,
  ProofTypes,
  ScreenshotProof,
  SignatureProof,
  Status,
  Theme,
  Transaction,
  TransactionAsset,
  VASP,
  Wallet,
  Withdrawal,
} from './types';
import { CMType, TransactionResponse } from './types';
import { encodeObjectToFragment } from './utils/urls';

export type {
  CMType,
  ComponentMessage,
  EmbeddedComponent,
  HostMessage,
  Theme,
  Counterparty,
  Withdrawal,
  Transaction,
  TransactionResponse,
  Destination as destination,
  TransactionAsset,
  SignatureProof,
  ScreenshotProof,
  MicroTransferProof,
  DeclarationProof,
  Notabene,
  ProofStatus,
  ProofTypes,
  IVMS101,
  PersonType,
  Status,
  VASP,
  Wallet,
  Agent,
  AgentType,
  OwnershipProof,
};

/**
 * Configuration for the Notabene SDK
 *
 * @public
 */
export interface NotabeneConfig {
  nodeUrl?: string;
  authToken?: string;
  uxUrl?: string;
  theme?: Theme;
  dictionary?: { [key: string]: string };
}

/**
 * Primary constructor for Notabene UX elements
 *
 * @public
 */

export default class Notabene {
  private nodeUrl?: string;
  private authToken?: string;
  private uxUrl: string;
  private theme?: Theme;
  private dictionary?: { [key: string]: string };

  constructor(config: NotabeneConfig) {
    this.uxUrl = config.uxUrl || 'https://connect.notabene.id';
    this.nodeUrl = config.nodeUrl;
    this.authToken = config.authToken;
    this.theme = config.theme;
    this.dictionary = config.dictionary;
  }

  public componentUrl(
    path: string,
    value: Partial<Transaction>,
    configuration?: any,
    callbacks?: CallbackOptions,
  ): string {
    const url = new URL(this.uxUrl);
    url.pathname = path;

    const hash = encodeObjectToFragment({
      authToken: this.authToken,
      value,
      configuration,
    });
    url.hash = hash;
    if (this.nodeUrl) url.searchParams.set('nodeUrl', this.nodeUrl);

    if (this.theme) {
      url.searchParams.set('theme', JSON.stringify(this.theme));
    }
    if (this.dictionary) {
      url.searchParams.set('dictionary', JSON.stringify(this.dictionary));
    }
    if (callbacks) {
      if (callbacks.callback)
        url.searchParams.set('callback_url', callbacks.callback);
      if (callbacks.redirectUri)
        url.searchParams.set('redirect_uri', callbacks.redirectUri);
    }
    return url.toString();
  }

  public createComponent(
    path: string,
    value: Partial<Transaction>,
    options?: any,
    callbacks?: CallbackOptions,
  ) {
    return new EmbeddedComponent(
      this.componentUrl(path, value, options, callbacks),
      value,
    );
  }

  public createWithdrawalAssist(
    value: Partial<Withdrawal>,
    options?: any,
    callbacks?: CallbackOptions,
  ) {
    return this.createComponent('withdrawal-assist', value, options, callbacks);
  }

  public decodeFragmentToObject(fragment: string): Record<string, string> {
    const pairs = fragment.slice(1).split('&');
    return pairs.reduce(
      (obj, pair) => {
        const [key, value] = pair.split('=');
        obj[decodeURIComponent(key)] = decodeURIComponent(value);
        return obj;
      },
      {} as Record<string, string>,
    );
  }

  // private openModal(url: string): void {
  //   if (this.modal) {
  //     this.closeModal();
  //   }
  //   this.modal = document.createElement('div');
  //   this.modal.style.position = 'fixed';
  //   this.modal.style.top = '0';
  //   this.modal.style.left = '0';
  //   this.modal.style.width = '100%';
  //   this.modal.style.height = '100%';
  //   this.modal.style.backgroundColor = 'rgba(0, 0, 0, 0.5)';
  //   this.modal.style.display = 'flex';
  //   this.modal.style.justifyContent = 'center';
  //   this.modal.style.alignItems = 'center';
  //   const iframe = document.createElement('iframe');
  //   iframe.src = url;
  //   iframe.setAttribute('allowtransparency', 'true');
  //   // iframe.allowtransparency = true;
  //   iframe.style.backgroundColor = 'transparent';
  //   iframe.style.width = '90%';
  //   iframe.style.height = '90%';
  //   iframe.style.border = 'none';
  //   iframe.style.overflow = 'hidden';
  //   this.modal.appendChild(iframe);
  //   document.body.appendChild(this.modal);
  //   this.modal.addEventListener('click', () => {
  //     console.log('click');
  //     this.closeModal();
  //   });
  //   document.addEventListener('keydown', ({ key }) => {
  //     console.log(`key: ${key}`);
  //     if (key === 'Escape') {
  //       console.log('esc');
  //       this.closeModal();
  //     }
  //   });
  // }

  // private closeModal(): void {
  //   if (this.modal) {
  //     console.log('this.closeModal');
  //     this.modal.remove();
  //     this.modal = undefined;
  //   }
  // }
}
